package java.io;

import java.util.Arrays;

public class StreamTokenizer
{
    private Reader reader;
    private InputStream input;
    private char[] buf;
    private int peekc;
    private static final int NEED_CHAR = Integer.MAX_VALUE;
    private static final int SKIP_LF = 2147483646;
    private boolean pushedBack;
    private boolean forceLower;
    private int LINENO;
    private boolean eolIsSignificantP;
    private boolean slashSlashCommentsP;
    private boolean slashStarCommentsP;
    private byte[] ctype;
    private static final byte CT_WHITESPACE = 1;
    private static final byte CT_DIGIT = 2;
    private static final byte CT_ALPHA = 4;
    private static final byte CT_QUOTE = 8;
    private static final byte CT_COMMENT = 16;
    public int ttype;
    public static final int TT_EOF = -1;
    public static final int TT_EOL = 10;
    public static final int TT_NUMBER = -2;
    public static final int TT_WORD = -3;
    private static final int TT_NOTHING = -4;
    public String sval;
    public double nval;
    
    private StreamTokenizer() {
        this.reader = null;
        this.input = null;
        this.buf = new char[20];
        this.peekc = Integer.MAX_VALUE;
        this.LINENO = 1;
        this.eolIsSignificantP = false;
        this.slashSlashCommentsP = false;
        this.slashStarCommentsP = false;
        this.ctype = new byte[256];
        this.ttype = -4;
        this.wordChars(97, 122);
        this.wordChars(65, 90);
        this.wordChars(160, 255);
        this.whitespaceChars(0, 32);
        this.commentChar(47);
        this.quoteChar(34);
        this.quoteChar(39);
        this.parseNumbers();
    }
    
    @Deprecated
    public StreamTokenizer(InputStream input) {
        this();
        if (input == null) {
            throw new NullPointerException();
        }
        this.input = input;
        return;
    }
    
    public StreamTokenizer(Reader reader) {
        this();
        if (reader == null) {
            throw new NullPointerException();
        }
        this.reader = reader;
        return;
    }
    
    public void resetSyntax() {
        int length = this.ctype.length;
        while (--length >= 0) {
            this.ctype[length] = 0;
        }
    }
    
    public void wordChars(int i, int n) {
        if (i < 0) {
            i = 0;
        }
        if (n >= this.ctype.length) {
            n = this.ctype.length - 1;
        }
        while (i <= n) {
            byte[] ctype = this.ctype;
            int n2 = i++;
            ctype[n2] |= 0x4;
        }
    }
    
    public void whitespaceChars(int i, int n) {
        if (i < 0) {
            i = 0;
        }
        if (n >= this.ctype.length) {
            n = this.ctype.length - 1;
        }
        while (i <= n) {
            this.ctype[i++] = 1;
        }
    }
    
    public void ordinaryChars(int i, int n) {
        if (i < 0) {
            i = 0;
        }
        if (n >= this.ctype.length) {
            n = this.ctype.length - 1;
        }
        while (i <= n) {
            this.ctype[i++] = 0;
        }
    }
    
    public void ordinaryChar(int n) {
        if (n >= 0 && n < this.ctype.length) {
            this.ctype[n] = 0;
        }
    }
    
    public void commentChar(int n) {
        if (n >= 0 && n < this.ctype.length) {
            this.ctype[n] = 16;
        }
    }
    
    public void quoteChar(int n) {
        if (n >= 0 && n < this.ctype.length) {
            this.ctype[n] = 8;
        }
    }
    
    public void parseNumbers() {
        for (int i = 48; i <= 57; ++i) {
            byte[] ctype = this.ctype;
            int n = i;
            ctype[n] |= 0x2;
        }
        byte[] ctype2 = this.ctype;
        int n2 = 46;
        ctype2[n2] |= 0x2;
        byte[] ctype3 = this.ctype;
        int n3 = 45;
        ctype3[n3] |= 0x2;
    }
    
    public void eolIsSignificant(boolean eolIsSignificantP) {
        this.eolIsSignificantP = eolIsSignificantP;
    }
    
    public void slashStarComments(boolean slashStarCommentsP) {
        this.slashStarCommentsP = slashStarCommentsP;
    }
    
    public void slashSlashComments(boolean slashSlashCommentsP) {
        this.slashSlashCommentsP = slashSlashCommentsP;
    }
    
    public void lowerCaseMode(boolean forceLower) {
        this.forceLower = forceLower;
    }
    
    private int read() throws IOException {
        if (this.reader != null) {
            return this.reader.read();
        }
        if (this.input != null) {
            return this.input.read();
        }
        throw new IllegalStateException();
    }
    
    public int nextToken() throws IOException {
        if (this.pushedBack) {
            this.pushedBack = false;
            return this.ttype;
        }
        byte[] ctype = this.ctype;
        this.sval = null;
        int n = this.peekc;
        if (n < 0) {
            n = Integer.MAX_VALUE;
        }
        if (n == 2147483646) {
            n = this.read();
            if (n < 0) {
                return this.ttype = -1;
            }
            if (n == 10) {
                n = Integer.MAX_VALUE;
            }
        }
        if (n == Integer.MAX_VALUE) {
            n = this.read();
            if (n < 0) {
                return this.ttype = -1;
            }
        }
        this.ttype = n;
        this.peekc = Integer.MAX_VALUE;
        int n2 = (n < 256) ? ctype[n] : 4;
        while ((n2 & 0x1) != 0x0) {
            if (n == 13) {
                ++this.LINENO;
                if (this.eolIsSignificantP) {
                    this.peekc = 2147483646;
                    return this.ttype = 10;
                }
                n = this.read();
                if (n == 10) {
                    n = this.read();
                }
            }
            else {
                if (n == 10) {
                    ++this.LINENO;
                    if (this.eolIsSignificantP) {
                        return this.ttype = 10;
                    }
                }
                n = this.read();
            }
            if (n < 0) {
                return this.ttype = -1;
            }
            n2 = ((n < 256) ? ctype[n] : 4);
        }
        if ((n2 & 0x2) != 0x0) {
            boolean n6 = false;
            if (n == 45) {
                n = this.read();
                if (n != 46 && (n < 48 || n > 57)) {
                    this.peekc = n;
                    return this.ttype = 45;
                }
                n6 = true;
            }
            double n7 = 0.0;
            int i = 0;
            int n8 = 0;
            while (true) {
                if (n == 46 && n8 == 0) {
                    n8 = 1;
                }
                else if (48 <= n && n <= 57) {
                    n7 = n7 * 10.0 + (double)(n - 48);
                    i += n8;
                }
                else {
                    break;
                }
                n = this.read();
            }
            this.peekc = n;
            if (i != 0) {
                double n9 = 10.0;
                --i;
                while (i > 0) {
                    n9 *= 10.0;
                    --i;
                }
                n7 /= n9;
            }
            this.nval = (n6 ? (-n7) : n7);
            return this.ttype = -2;
        }
        else if ((n2 & 0x4) != 0x0) {
            int count2 = 0;
            do {
                if (count2 >= this.buf.length) {
                    this.buf = Arrays.copyOf(this.buf, this.buf.length * 2);
                }
                this.buf[count2++] = (char)n;
                n = this.read();
            } while ((((n < 0) ? 1 : ((n < 256) ? ctype[n] : 4)) & 0x6) != 0x0);
            this.peekc = n;
            this.sval = String.copyValueOf(this.buf, 0, count2);
            if (this.forceLower) {
                this.sval = this.sval.toLowerCase();
            }
            return this.ttype = -3;
        }
        else {
            if ((n2 & 0x8) != 0x0) {
                this.ttype = n;
                int count;
                int n5;
                int read5;
                for (count = 0, n5 = this.read(); n5 >= 0 && n5 != this.ttype && n5 != 10 && n5 != 13; this.buf[count++] = (char)read5) {
                    if (n5 == 92) {
                        read5 = (read5 = this.read());
                        if (read5 >= 48 && read5 <= 55) {
                            read5 -= 48;
                            int read6 = this.read();
                            if (48 <= read6 && read6 <= 55) {
                                read5 = (read5 << 3) + (read6 - 48);
                                int read7 = this.read();
                                if (48 <= read7 && read7 <= 55 && read5 <= 51) {
                                    read5 = (read5 << 3) + (read7 - 48);
                                    n5 = this.read();
                                }
                                else {
                                    n5 = read7;
                                }
                            }
                            else {
                                n5 = read6;
                            }
                        }
                        else {
                            switch (read5) {
                                case 97: {
                                    read5 = 7;
                                    break;
                                }
                                case 98: {
                                    read5 = 8;
                                    break;
                                }
                                case 102: {
                                    read5 = 12;
                                    break;
                                }
                                case 110: {
                                    read5 = 10;
                                    break;
                                }
                                case 114: {
                                    read5 = 13;
                                    break;
                                }
                                case 116: {
                                    read5 = 9;
                                    break;
                                }
                                case 118: {
                                    read5 = 11;
                                    break;
                                }
                            }
                            n5 = this.read();
                        }
                    }
                    else {
                        read5 = n5;
                        n5 = this.read();
                    }
                    if (count >= this.buf.length) {
                        this.buf = Arrays.copyOf(this.buf, this.buf.length * 2);
                    }
                }
                this.peekc = ((n5 == this.ttype) ? Integer.MAX_VALUE : n5);
                this.sval = String.copyValueOf(this.buf, 0, count);
                return this.ttype;
            }
            if (n == 47 && (this.slashSlashCommentsP || this.slashStarCommentsP)) {
                int read2 = this.read();
                if (read2 == 42 && this.slashStarCommentsP) {
                    int n3 = 0;
                    int n4;
                    while ((n4 = this.read()) != 47 || n3 != 42) {
                        if (n4 == 13) {
                            ++this.LINENO;
                            n4 = this.read();
                            if (n4 == 10) {
                                n4 = this.read();
                            }
                        }
                        else if (n4 == 10) {
                            ++this.LINENO;
                            n4 = this.read();
                        }
                        if (n4 < 0) {
                            return this.ttype = -1;
                        }
                        n3 = n4;
                    }
                    return this.nextToken();
                }
                if (read2 == 47 && this.slashSlashCommentsP) {
                    int read4;
                    while ((read4 = this.read()) != 10 && read4 != 13 && read4 >= 0) {}
                    this.peekc = read4;
                    return this.nextToken();
                }
                if ((ctype[47] & 0x10) != 0x0) {
                    int read3;
                    while ((read3 = this.read()) != 10 && read3 != 13 && read3 >= 0) {}
                    this.peekc = read3;
                    return this.nextToken();
                }
                this.peekc = read2;
                return this.ttype = 47;
            }
            else {
                if ((n2 & 0x10) != 0x0) {
                    int read;
                    while ((read = this.read()) != 10 && read != 13 && read >= 0) {}
                    this.peekc = read;
                    return this.nextToken();
                }
                return this.ttype = n;
            }
        }
    }
    
    public void pushBack() {
        if (this.ttype != -4) {
            this.pushedBack = true;
        }
    }
    
    public int lineno() {
        return this.LINENO;
    }
    
    @Override
    public String toString() {
        String str;
        switch (this.ttype) {
            case -1: {
                str = "EOF";
                break;
            }
            case 10: {
                str = "EOL";
                break;
            }
            case -3: {
                str = this.sval;
                break;
            }
            case -2: {
                str = "n=" + this.nval;
                break;
            }
            case -4: {
                str = "NOTHING";
                break;
            }
            default: {
                if (this.ttype < 256 && (this.ctype[this.ttype] & 0x8) != 0x0) {
                    str = this.sval;
                    break;
                }
                char[] value = new char[3];
                value[0] = (value[2] = '\'');
                value[1] = (char)this.ttype;
                str = new String(value);
                break;
            }
        }
        return "Token[" + str + "], line " + this.LINENO;
    }
}
